#!/usr/bin/python

# ----------------------------------------------------------------------------
#
#  Copyright (C) 2008-2021 Fons Adriaensen <fons@linuxaudio.org>
#    
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http:#www.gnu.org/licenses/>.
#
# ----------------------------------------------------------------------------


# This example shows how to resample a signal such that
# - the input and output have the same duration,
# - the first sample of the output corresponds to
#   the first sample of the input.
# To do this, it is necessary to add some zeros before
# and after the input. This can be done explicitly which
# requires a copy of the input, or in the way shown below.


import numpy as np
import matplotlib.pyplot as plt
from zita_audiotools.vresampler import *

# Upsampling ratio is 3.2, single channel.
#
R = VResampler (3.2, 1, 32)

# Input and output arrays.
#
X = np.zeros ((200,), dtype = np.float32)
Y = np.zeros ((600,), dtype = np.float32)

# We put single sample impulses at positions 0 and
# 100 of the input, so we expect to see a sinc
# function at positions 0 and 320 in the output.
#
X [0] = 1
X [100] = 1

k = R.inpsize () # Size of the polyphase FIR filter.

# Insert k // 2 - 1 zeros before X.
#
ni, no = R.process (k // 2 - 1, Y)
assert (ni == k // 2 - 1)  # All zero samples used.
assert (no == 0)           # Nothing written to Y.

# Resample all of X.
#
ni, no = R.process (X, Y)
assert (ni == X.shape [0]) # All of X is used.
assert (no < Y.shape [0])  # Not all of Y filled.

# Insert k // 2 zeros after X.
#
ni, no = R.process (k // 2, Y [no:])

# Plot input and result.
#
fig = plt.figure (figsize=(8,6), facecolor='white')
ax1 = fig.add_axes ([0.07, 0.54, 0.90, 0.43])
ax2 = fig.add_axes ([0.07, 0.05, 0.90, 0.43])
ax1.set_ylim (-1.2, 1.2)
ax2.set_ylim (-1.2, 1.2)
ax1.plot (X)
ax2.plot (Y)
ax1.grid ()
ax2.grid ()
plt.show()
